@@ -22,53 +22,40 @@ from dnstap_receiver import dnstap as dnstap_pb2
 # framestreams decoder
 from dnstap_receiver import fstrm
 
-parser = argparse.ArgumentParser()
-parser.add_argument("-l", help="receive dnstap payloads from remote tcp sender, listen on ip:port")
-parser.add_argument("-u", help="read dnstap payloads using framestreams from unix socket")
-parser.add_argument('-v', action='store_true', help="verbose mode")
-parser.add_argument("-y", help="write YAML-formatted output", action='store_true')
-parser.add_argument("-j", help="write JSON-formatted output", action='store_true')                       
-parser.add_argument("-d", help="send dnstap message to remote tcp/ip address")   
-
-DNSTAP_TYPE = { 1: 'AUTH_QUERY',
-                2: 'AUTH_RESPONSE',
-                3: 'RESOLVER_QUERY',
-                4: 'RESOLVER_RESPONSE',
-                5: 'CLIENT_QUERY',
-                6: 'CLIENT_RESPONSE',
-                7: 'FORWARDER_QUERY',
-                8: 'FORWARDER_RESPONSE',
-                9: 'STUB_QUERY',
-                10: 'STUB_RESPONSE',
-                11: 'TOOL_QUERY',
-                12: 'TOOL_RESPONSE' }
+DNSTAP_TYPE = { 1: 'AUTH_QUERY', 2: 'AUTH_RESPONSE',
+                3: 'RESOLVER_QUERY', 4: 'RESOLVER_RESPONSE',
+                5: 'CLIENT_QUERY', 6: 'CLIENT_RESPONSE',
+                7: 'FORWARDER_QUERY', 8: 'FORWARDER_RESPONSE',
+                9: 'STUB_QUERY', 10: 'STUB_RESPONSE',
+                11: 'TOOL_QUERY', 2: 'TOOL_RESPONSE' }
 DNSTAP_FAMILY = {1: 'IP4', 2: 'IP6'}
 DNSTAP_PROTO = {1: 'UDP', 2: 'TCP'}    
 
 DATETIME_FORMAT ='%Y-%m-%d %H:%M:%S.%f'
 
-FMT_SHORT = "SHORT"
-FMT_JSON = "JSON"
-FMT_YAML = "YAML"
-
-try:
-    args = parser.parse_args()
-except:
-    sys.exit(1)
-
-# init logging
-level = logging.INFO
-if args.v:
-    level = logging.DEBUG
-logging.basicConfig(format='%(asctime)s %(message)s', level=level)
+# command line arguments definition
+parser = argparse.ArgumentParser()
+parser.add_argument("-l", 
+                    help="IP of the dnsptap server to receive dnstap payloads (default: %(default)r)",
+                    default="0.0.0.0")
+parser.add_argument("-p", type=int,
+                    help="Port the dnstap receiver is listening on (default: %(default)r)",
+                    default=6000)               
+parser.add_argument("-u", help="read dnstap payloads from unix socket")
+parser.add_argument('-v', action='store_true', help="verbose mode")   
+parser.add_argument("-c", help="external config file")   
 
 
-async def cb_ondnstap(dnstap_decoder, payload, tcp_writer, output_fmt):
+async def cb_ondnstap(dnstap_decoder, payload, tcp_writer, cfg):
     """on dnstap"""
     # decode binary payload
     dnstap_decoder.parse_from_bytes(payload)
     dm = dnstap_decoder.message
     
+    if "dnstap-identities" in cfg["input-mode"]:
+        if dnstap_decoder.identity.decode() not in cfg["input-mode"]["dnstap-identities"]:
+            return
+            
     tap = { "identity": dnstap_decoder.identity.decode(),
             "query-name": "-", 
             "query-type": "-", 
@@ -113,26 +100,28 @@ async def cb_ondnstap(dnstap_decoder, payload, tcp_writer, output_fmt):
     
     
     # reformat dnstap message
-    if output_fmt == FMT_SHORT:
+    if cfg["output-format"]["text"]:
         msg = "%s %s %s %s %s %s %s %s %sb %s %s" % (tap["timestamp"], tap["identity"], 
-                                               tap["message"], tap["code"],
-                                               tap["source-ip"], tap["source-port"],
-                                               tap["protocol"], tap["transport"], tap["length"],
-                                               tap["query-name"], tap["query-type"])
+                                                   tap["message"], tap["code"],
+                                                   tap["source-ip"], tap["source-port"],
+                                                   tap["protocol"], tap["transport"],
+                                                   tap["length"],
+                                                   tap["query-name"], tap["query-type"])
         
-    if output_fmt == FMT_JSON:
+    if cfg["output-format"]["json"]:
         msg = json.dumps(tap)
         
-    if output_fmt == FMT_YAML:
+    if cfg["output-format"]["yaml"]:
         msg = yaml.dump(tap)
 
-    # final step, stdout or remote destination ? send json message to remote tcp
+    # final step, stdout or remote destination ? 
+    # send json message to remote tcp
     if tcp_writer is not None:
         tcp_writer.write(msg.encode() + b"\n")
     else:
         print(msg)
 
-async def cb_onconnect(reader, writer):
+async def cb_onconnect(reader, writer, cfg):
     """callback when a connection is established"""
     sock = writer.get_extra_info('socket')
     if sock.family == socket.AF_UNIX:
@@ -146,23 +135,12 @@ async def cb_onconnect(reader, writer):
     loop = asyncio.get_event_loop()
     dnstap_decoder = dnstap_pb2.Dnstap()
     
-    args = parser.parse_args()
-    if args.d is None:
-        tcp_writer = None
-    else:
-        if ":" not in args.d:
-            raise Exception("bad remote ip provided")
-        dest_ip, dest_port = args.d.split(":", 1)
-        
-        _, tcp_writer = await asyncio.open_connection(dest_ip, 
-                                                      int(dest_port),
+    # remote connection to enable ?
+    tcp_writer = None
+    if cfg["forward-to"]["enable"]:
+        _, tcp_writer = await asyncio.open_connection(cfg["forward-to"]["remote-address"], 
+                                                      cfg["forward-to"]["remote-port"],
                                                       loop=loop)
-    
-    fmt = FMT_SHORT
-    if args.y:
-        fmt = FMT_YAML
-    if args.j:
-        fmt = FMT_JSON
 
     try:
         while data := await reader.read(fstrm_handler.pending_nb_bytes()) :
@@ -176,7 +154,7 @@ async def cb_onconnect(reader, writer):
 
                 # handle the DATA frame
                 if fs == fstrm.FSTRM_DATA_FRAME:
-                    loop.create_task(cb_ondnstap(dnstap_decoder, payload, tcp_writer, fmt))
+                    loop.create_task(cb_ondnstap(dnstap_decoder, payload, tcp_writer, cfg))
                     
                 # handle the control frame READY
                 if fs == fstrm.FSTRM_CONTROL_READY:
@@ -196,38 +174,68 @@ async def cb_onconnect(reader, writer):
                     logging.debug(f"{peername} - control stop received")
                     fstrm_handler.reset()           
     except asyncio.CancelledError:
-        print(f'{peername} closing connection.')
+        logging.debug(f'{peername} - closing connection.')
         writer.close()
         await writer.wait_closed()
     except asyncio.IncompleteReadError:
-        print(f'{peername} disconnected')
+        logging.debug(f'{peername} - disconnected')
     finally:
-        print(f'{peername} closed')
+        logging.debug(f'{peername} - closed')
 
-    # close the remote tcp conn
+    # close the remote connection
     if tcp_writer is not None:
         tcp_writer.close()
 
 def start_receiver():
     """start dnstap receiver"""
+    # Handle command-line arguments.
     args = parser.parse_args()
-    logging.debug("Start dnstap receiver...")
 
+    # external config file provided ?
+    if args.c:
+        try:
+            with open(args.c) as file:
+                cfg = yaml.safe_load(file)
+        except FileNotFoundError:
+            print("error: config file not found")
+            sys.exit(1)
+    else:
+        cfg = {"verbose": args.v, "input-mode": {}, 
+               "output-format": {} , "forward-to": {}}
+        cfg["input-mode"]["unix-socket"] = args.u
+        cfg["input-mode"]["local-address"] =  args.l
+        cfg["input-mode"]["local-port"] =  args.p
+        cfg["output-format"]["yaml"] = args.y
+        cfg["output-format"]["json"] =args.j
+        cfg["output-format"]["text"] =True
+        cfg["forward-to"]["enable"] =  False
+        cfg["forward-to"]["remote-port"] = None
+        cfg["forward-to"]["remote-port"] = None
+
+    # init logging
+    level = logging.INFO
+    if cfg["verbose"]:
+        level = logging.DEBUG
+    logging.basicConfig(format='%(asctime)s %(message)s', level=level)
+
+    logging.debug("Start receiver...")
     loop = asyncio.get_event_loop()
 
     # asynchronous unix socket
-    if args.u is not None:
-        socket_server = asyncio.start_unix_server(cb_onconnect, path=args.u, loop=loop)
-    elif args.l is not None:
-        if ":" not in args.l:
-            logging.error("malformed listen ip/port provided")
-            sys.exit(1)
-        listen_ip, listen_port = args.l.split(":", 1)
-        socket_server = asyncio.start_server(cb_onconnect, listen_ip, listen_port, loop=loop)
+    if cfg["input-mode"]["unix-socket"] is not None:
+        logging.debug("Listening on %s" % args.u)
+        socket_server = asyncio.start_unix_server(cb_onconnect,
+                                                  path=cfg["input-mode"]["unix-socket"],
+                                                  loop=loop)
+    # asynchronous tcp socket
     else:
-        logging.error("no input provided")
-        sys.exit(1)
-        
+        logging.debug("Listening on %s:%s" % (cfg["input-mode"]["local-address"],
+                                              cfg["input-mode"]["local-port"])), 
+        socket_server = asyncio.start_server(lambda r, w: cb_onconnect(r, w, cfg),
+                                             cfg["input-mode"]["local-address"],
+                                             cfg["input-mode"]["local-port"],
+                                             loop=loop)
+
     # run until complete
     loop.run_until_complete(socket_server)
     
